'This example is similar to one given in the HP 3457A Multimeter
'Operating manual in the 'triggering examples' section.
'
'Here is a description of the situation taken from the HP3457A manual:
'
'Programming example - "You want to digitize the noise and pulses present
'during the settling time of a positional control process.  The settling
'time is less than 50ms.  The pulse that starts the positional control
'also initiates the readings for 50ms.  The HP 3457 must be totally
'dedicated to this process.  At the same time, the system controller must
'be free for other computation.  You would like audio indication when the
'HP 3457's memory is full and an interrupt to initiate rapid data
'transfer.  After the data transfer is complete, the entire process will
'be repeated.
'
'For this application, you must maximize efficiency.  To do this, you
'will use the input buffer, subprogram storage, and the SRQ interrupt.
'You must create a routine for your controller that responds to an SRQ
'interrupt by checking the status byte.  This ensures that the program
'execution complete bit caused the SRQ.  After verifying the status byte,
'the interrupt routine should enter all stored readings, and manupulate
'and archive the received data."
'
'We will change the example slightly by simulating the control process
'using an HP 3325A synthesizer/function generator.  Instead of triggering
'the DVM on its external trigger line, we will set triggering to AUTO and
'let it go.  Instead of manipulating and archiving the data, we graph
'it on the controllers screen.  Once everything is set up, the controller
'is free to do other work, but this example program just sits in an idle
'loop.
'
'This example also shows how to use the parallel poll feature of the HPIB
'command library.  Before the SRQ.HANDLER routine is finished, it should
'make sure that the SRQ line is low, or else the interrupt routine will
'not get called again.  Use of the parallel poll feature simplifies this
'process.
'
'
REM $INCLUDE: 'QBSETUP'
'
ISC& = 7		    'interface select code
SOURCE& = ISC& * 100 + 12   'address of the function generator
DVM& = ISC& * 100 + 22	    'address of the digital volt meter
DIM READINGS%[50]	    'place we will put the readings from the dvm
'
'the F1 key will end the program
'
ON KEY(1) GOSUB QUIT
KEY(1) ON
'
'reset the HPIB interface
'
CALL IORESET(ISC&)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'set up a timeout of 5 seconds
'
TIME = 5
CALL IOTIMEOUT(ISC&, TIME)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'clear the devices we are going to use
'
CALL IOCLEAR(SOURCE&)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
CALL IOCLEAR(DVM&)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'we meet the minimum requirements for IOFASTOUT, so let's use it
'
CALL IOFASTOUT(ISC&, TRUE%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'we will now configure all devices that can respond to a parallel poll
' this example assumes devices at addresses 20 and 7 can respond to a
' parallel poll.  see operators manual of individual devices to see if
' they can respond to a parallel poll.
'
'configure the device at address 20 for a parallel poll
'
DEVICE.ADDR& = ISC& * 100 + 20
CONFIGURATION% = &H08 'RESPOND WITH A "1" ON LINE 0
CALL IOPPOLLC(DEVICE.ADDR&, CONFIGURATION%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'configure the device at address 7 for a parallel poll
'
DEVICE.ADDR& = ISC& * 100 + 7
CONFIGURATION% = &H09 'RESPOND WITH A "1" ON LINE 1
CALL IOPPOLLC(DEVICE.ADDR&, CONFIGURATION%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'configure any other devices that can respond to parallel poll here
'
'let's use dma to send the strings to program the devices
'
COUNT% = 40: CHANNEL% = 3
CALL IODMA(ISC&, COUNT%, CHANNEL%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'program the function generator
'
CODES$ = "RF1 FR4KH FU1 ST1KH SP10KH MF1KH AM1VR TI5SE"
LENGTH% = LEN(CODES$)
CALL IOOUTPUTS(SOURCE&, CODES$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'program the dvm
'in a real application, the TRIG AUTO would be replaced with a TRIG EXT
'
CODES$ = "INBUF ON;DCV 30,.1;NPLC 0;AZERO OFF;DELAY 0;DISP OFF;LOCK ON;"
CODES$ = CODES$ + "EMASK 2047;RQS 105;TARM HOLD;MSIZE 1000,150;MEM FIFO;"
CODES$ = CODES$ + "MFORMAT SINT;OFORMAT SINT;TRIG AUTO;NRDGS 50,AUTO;"
CODES$ = CODES$ + "SUB 1;TARM SGL,10;BEEP ONCE;SUBEND;CALL 1"
LENGTH% = LEN(CODES$)
CALL IOOUTPUTS (DVM&, CODES$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'turn dma off again
'
COUNT% = 0: CHANNEL% = 3
CALL IODMA(ISC&, COUNT%, CHANNEL%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'turn on automatic srq checking
'
ON PEN GOSUB SRQ.HANDLER
PEN ON
'
PRIORITY% = 0
CALL IOPEN(ISC&, PRIORITY%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'
BACK:
    '
    'can have controller do other work here
    '
    GOTO BACK
'
'end of main program.  support routines follow.
'
'
'this is the routine defined by the 'on pen gosub' statement above.
' this routine will be called each time an srq comes in.
'
SRQ.HANDLER:
    '
    'do a serial poll of the dvm first, since this is the most likely
    ' source of the srq
    '
    CALL IOSPOLL(DVM&, RESPONSE%)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    IF (RESPONSE% AND 32) <> 0 THEN GOTO DVM.ERROR
    IF (RESPONSE% AND 1) = 0 THEN GOTO NO.DVM.SRQ
    '
    'the dvm is ready to send us some data
    '
    'enter the data
    '
    FLAG% = 2 : MAX% = 50*FLAG% : ACTUAL% = 0
    CALL IOENTERB (DVM&, READINGS%[0], MAX%, ACTUAL%, FLAG%)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    ACTUAL% = ACTUAL% / FLAG%
    '
    'at this point, the controller can do anything to the data -
    ' manipulate it, save it, etc.  this particular programs graphs
    ' the data.
    '
    GOSUB GRAPH.DATA
    '
  NO.DVM.SRQ:
    '
    'need make sure that srq is low here, or no other srq's will be
    ' detected.
    '
    STAT% = 1: RESPONSE% = 0
    CALL IOSTATUS(ISC&, STAT%, RESPONSE%)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    IF RESPONSE% = 0 THEN GOTO END.HANDLER
	'
	'conduct a parallel poll
	'note that the source doesn't respond to parallel poll's, so we
	' need to poll that device separately.
	'
	RESPONSE% = 0
	CALL IOPPOLL(ISC&, RESPONSE%)
	IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
	IF (RESPONSE% AND 1) THEN GOSUB POLL.DEVICE.1
	IF (RESPONSE% AND 2) THEN GOSUB POLL.DEVICE.2
	'
	'check all devices that were configured to respond to parallel
	' poll
	'
	'check any other devices on the bus here that weren't configured
	' to respond to parallel poll by performing a serial poll on each
	' one.
	'
	CALL IOSPOLL(SOURCE&, RESPONSE%)
	IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
	'
	'see if we've cleared the srq yet
	'
	STAT% = 1: RESPONSE% = 0
	CALL IOSTATUS(ISC&, STAT%, RESPONSE%)
	IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
	IF RESPONSE% = 1 THEN PRINT "SRQ LOCKED HIGH" : GOTO QUIT
	'
  END.HANDLER:
    '
    'get the dvm going again
    '
    CODES$ = "CALL 1"
    LENGTH% = LEN(CODES$)
    CALL IOOUTPUTS (DVM&, CODES$, LENGTH%)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    '
RETURN
'
'
POLL.DEVICE.1:
    '
    'do a serial poll of the device configured to use parallel poll
    ' line 0
    '
    DEVICE.ADDR& = ISC& * 100 + 20
    CALL IOSPOLL(DEVICE.ADDR&, RESPONSE%)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    '
    'should check RESPONSE% here to see if any action needs to be taken.
    'the values that RESPONSE% can take are device dependent.
    '
    RETURN
'
'
POLL.DEVICE.2:
    '
    'do a serial poll of the device configured to use parallel poll
    ' line 1
    '
    DEVICE.ADDR& = ISC& * 100 + 7
    CALL IOSPOLL(DEVICE.ADDR&, RESPONSE%)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    '
    'should check RESPONSE% here to see if any action needs to be taken.
    'the values that RESPONSE% can take are device dependent.
    '
    RETURN
'
'
GRAPH.DATA:
    '
    'set graphics mode and draw border on screen
    '
    CLS: SCREEN 2
    WINDOW (0,0)-(639,199)
    LINE (0,0)-(639,199),,B
    '
    'calculate min and max values for y-axis
    '
    YMAX = READINGS%[0] : YMIN = READINGS%[0]
    FOR I = 1 TO ACTUAL%
	IF READINGS%[I] < YMIN THEN YMIN = READINGS%[I]
	IF READINGS%[I] > YMAX THEN YMAX = READINGS%[I]
    NEXT I
    '
    'print graph labels
    '
    LOCATE 2,2 : PRINT "MAX = ";YMAX;
    LOCATE 24,2 : PRINT "MIN = ";YMIN;
    '
    'scale ymin and ymax so there is space between graph and border
    '
    IF YMIN > 0 THEN YMIN = YMIN*0.6 ELSE YMIN = YMIN*1.4
    IF YMAX > 0 THEN YMAX = YMAX*1.4 ELSE YMAX = YMAX*0.6
    '
    'graph the data
    '
    WINDOW (0,YMIN)-(ACTUAL%-1,YMAX)
    XAXIS = 1
    PREV = READINGS%[XAXIS-1]
    PSET(XAXIS-1, PREV)
    WHILE (XAXIS < ACTUAL%)
      CURRENT = READINGS%[XAXIS]
      LINE (XAXIS-1, PREV)-(XAXIS,CURRENT)
      PREV = CURRENT
      XAXIS = XAXIS + 1
    WEND
    RETURN
'
'
DVM.ERROR:
    '
    PRINT "THAT'S A DVM ERROR"
    '
    'we have an error, so let's abort all activity on the HPIB bus
    '
    CALL IOABORT(ISC&)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    '
    'clear the DVM so we can ask it for error information
    '
    CALL IOCLEAR(DVM&)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    '
    'ask the dvm what the error was
    '
    CODES$ = "ERR?"
    LENGTH% = LEN(CODES$)
    CALL IOOUTPUTS (DVM&, CODES$, LENGTH%)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    '
    'read the error value
    '
    CALL IOENTER (DVM&, ERROR.NUMBER)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    '
    'print the error message
    '
    PRINT "DVM ERROR NUMBER ";ERROR.NUMBER
    '
    'stop the program
    '
'
'
QUIT:
    '
    'clear the dvm so we can send the commands to reset it
    '
    CALL IOCLEAR(DVM&)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    '
    'reset the dvm
    '
    CODES$ = "LOCK OFF;DISPLAY ON;RESET"
    LENGTH% = LEN(CODES$)
    CALL IOOUTPUTS (DVM&, CODES$, LENGTH%)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    '
    'unconfigure the parallel poll
    '
    CALL IOPPOLLU(ISC&)
    IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
    '
    SYSTEM
