'This program requires two device to run:
'     54201A/D digitizing oscilloscope (or compatible scope)
'     a printer capable of printing in HP RASTER GRAPHICS STANDARD
'       (e.g. thinkjet)
'This program will tell the scope to take a reading on channel 1, then
'will send the data back to this program.  We could do anything we want
'to the data at this time, and we chose to print some simple statistics
'about the data.  The program then tells the scope to send the data
'directly to the printer, illustrating how the controller does not have to
'be directly involved in an HPIB transaction.  The program then sends the
'data back to the scope, where the scope displays it on graph 2.
'
'A couple of things to note about this program:
' - Note the use of the IOENTERB and IOOUTPUTB in conjunction with
'   IOENTERS and IOOUTPUTS respectively to simulate the IEEE #A format.
'   This format is as follows: An ascii '#A', then two bytes containing
'   the length of the data to follow, then the actual data.  Similar
'   usage of these functions can be used to simulate other IEEE formats.
'
' - If your device uses IEEE 488.2 arbitrary block data transfers, you
'   can use IOENTERAB and IOOUTPUTAB instead of combining IOENTERS with
'   IOENTERB or IOOUTPUTS with IOOUTPUTB to simulate the arbitrary block
'   format.
'
' - You should probably disable character matching before executing an
'   IOENTERB or IOENTERAB, as the character in the MATCH string is
'   generally a valid binary value, rather than a termination character.
'
' - The commands that are sent to the scope are device dependent, and are
'   found in the manual for the scope.
'
' - The error checking in the program consists of executing the line
'       IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'   after each call to an HPIB command.
'
' - Before an IOENTERS statement is executed, space should be allocated
'   for the string by assigning the string to SPACE$(n) where n is the
'   maximum length to be entered.  If this is not done, the string will
'   be of length 0, and no characters will be entered.  You can then
'   shorten the string to the correct length (in case less than the
'   maximum # of chars were entered) by using the LEFT$ function.
'
'this program has 4 main parts to it.  they are:
'   1. read the data from the scope
'   2. print some statistics about the data
'   3. have the scope send the data to a printer
'   4. send the data from the controller back to the scope and display
'      it on the scope's screen
'
REM $INCLUDE: 'QBSETUP'
ISC& = 7
SCOPE& = ISC& * 100 + 7
'
'reset the HPIB interface
'
CALL IORESET(ISC&)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'set up a timeout of 5 seconds
'
TIME! = 5
CALL IOTIMEOUT(ISC&, TIME!)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'clear the scope
'
CALL IOCLEAR(SCOPE&)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'lockout the keyboard on the scope so the program has complete control
' of it
'
CALL IOLLOCKOUT(ISC&)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'set up some variables
'
OPTION BASE 1
DISABLE% = 0 : ENABLE% = 1
EOL$ = CHR$(10)
CRLF$ = CHR$(13) + CHR$(10)
CMD$ = SPACE$(255)
'
'
'
'this is the beginning of part 1    ************************
'     - get the data from the scope
'
'
'setup scope to accept waveform data
'
CMD$ = "HEADER OFF;EOI ON;DISPLAY FORMAT 1"
LENGTH% = LEN(CMD$)
CALL IOOUTPUTS(SCOPE&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'acquire data
'
CALL IOTRIGGER(SCOPE&)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'setup up the waveform source
'
CMD$ = "WAVEFORM;SOURCE CHANNEL 1;FORMAT WORD"
LENGTH% = LEN(CMD$)
CALL IOOUTPUTS(SCOPE&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'input waveform preamble to controller
'
'the scope must be triggered before it will output the preamble
'
PRINT "WAITING FOR TRIGGER. . ."
'
'ask the scope to send the preamble
'
PRE$ = SPACE$(150)
CMD$ = "WAVEFORM PREAMBLE?"
LENGTH% = LEN(CMD$)
CALL IOOUTPUTS(SCOPE&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'read in the preamble
'
MAX% = 150: ACTUAL% = 0
CALL IOENTERS(SCOPE&, PRE$, MAX%, ACTUAL%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
PRE$ = LEFT$(PRE$, ACTUAL%)
'
'turn off 'lf' enter terminator, as 'lf' is a legal binary value
'
CALL IOMATCH(ISC&, EOL$, DISABLE%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'make sure EOI is set
'
CALL IOEOI(ISC&, ENABLE%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'command scope to send data.  It will be sent in IEEE #A format.
'
CMD$ = "DATA?"
LENGTH% = LEN(CMD$)
CALL IOOUTPUTS(SCOPE&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'Note that if your device can use IEEE 488.2 arbitrary block format, the
'   commands from here to right after the IOGETTERM command can be
'   replaced with one IOENTERAB command.
'
'first, enter the ASCII '#A' from the scope
'
MAX% = 2: ACTUAL% = 0: HEADER$ = SPACE$(2)
CALL IOENTERS(SCOPE&, HEADER$, MAX%, ACTUAL%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'check that the correct header was received
'
HEADER$ = LEFT$(HEADER$, ACTUAL%)
IF HEADER$ <> "#A" THEN PRINT "WRONG HEADER RECEIVED": STOP
'
'enter the number of points
'
MAX% = 2: BYTES% = 0: ACTUAL% = 0: FLAG% = 2
CALL IOENTERB(SCOPE&, SEG BYTES%, MAX%, ACTUAL%, FLAG%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
POINTS% = BYTES%/2
'
'enter the actual data
'
DIM READINGS%(5000): MAX% = BYTES%: ACTUAL% = 0: FLAG% = 2
CALL IOENTERB(SCOPE&, SEG READINGS%(1), MAX%, ACTUAL%, FLAG%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'use IOGETTERM to see if we entered all the points
'
CALL IOGETTERM(ISC&, REASON%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
IF (REASON% AND 1) = 0 THEN PRINT "NOT ALL POINTS FOUND"
'
'
'
'this is the beginning of part 2    **************************
'     - print some statistics about the data
'
'
'calculate minimum, maximum, and sum of the values in the data
'
MAX = READINGS%[1] : MIN = READINGS%[1] : AVE = 0
FOR I = 1 TO POINTS%
    AVE = AVE + READINGS%[I]
    IF READINGS%[I] < MIN THEN MIN = READINGS%[I]
    IF READINGS%[I] > MAX THEN MAX = READINGS%[I]
NEXT I
'
'print the stats
'
PRINT
PRINT " -------  statistics  --------"
PRINT "  Minumum value = "; MIN
PRINT "  Maximum value = "; MAX
PRINT "  Mean          = "; AVE/POINTS%
PRINT
'
'
'
'this is the beginning of part 3    ************************
'     - have the scope send the graph directly to the printer
'
'
'Now, lets tell the scope to print directly to a printer.
' We must first send the HPIB commands to make the scope a talker, and
' the printer a listener.  This will be done using the IOSEND command.
' We will tell the scope to issue a service request when its done
' printing, as we need to wait for the printing to complete before
' continuing the program.
'
'tell the scope to SRQ on 'hardcopy complete'
'
CMD$ = "RQS 4112"   'the 4112 is a device-dependent number.
LENGTH% = LEN(CMD$)
CALL IOOUTPUTS(SCOPE&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'tell scope to print
'
CMD$ = "PRINT"
LENGTH% = LEN(CMD$)
CALL IOOUTPUTS(SCOPE&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'tell scope to talk and printer to listen
' the listen command is formed by adding 32 to the device address of the
'     device to be a listener
' the talk command is formed by adding 64 to the device address of the
'     device to be a talker
'
PRINT.LISTEN$ = CHR$(32 + 1)   'printer is at device address 1
SCOPE.TALK$   = CHR$(64 + 7)   'scope is at device address 7
UNLISTEN$ = CHR$(63)
'
'send the command
'
CMD$ = UNLISTEN$ + PRINT.LISTEN$ + SCOPE.TALK$
LENGTH% = LEN(CMD$)
CALL IOSEND(ISC&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'now, the ATN line must be set to FALSE.
' To do this, we will do a dummy IOENTERS command with length = 0.
' This will cause an error, however, unless we first make the controller
' a listener.  We will do this with the IOCONTROL command.
'
'first, make controller a listener
'
COND% = 6
STATUS% = 1
CALL IOCONTROL(ISC&, COND%, STATUS%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'now do the dummy IOENTERS
'
MAX% = 0: ACTUAL% = 0: CMD$ = SPACE$(0)
CALL IOENTERS(ISC&, CMD$, MAX%, ACTUAL%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'now, make controller non-listener again
'
COND% = 6
STATUS% = 0
CALL IOCONTROL(ISC&, COND%, STATUS%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'now wait for SRQ from scope
'
NOT.DONE:
'
COND% = 1: STATUS% = 0
CALL IOSTATUS(ISC&, COND%, STATUS%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
IF STATUS% = 0 THEN GOTO NOT.DONE
'
'make sure it was the scope requesting service
'
CALL IOSPOLL(SCOPE&, STATUS%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
' 16 = bit 4 set = 'scope ready'
'
IF (STATUS% AND 16) = 0 THEN GOTO NOT.DONE
'
'we can continue now
'
'this is the beginning of part 4    *********************************
'     - send the data back to the scope to be displayed on graph 2
'
'
'tell the scope where to put the data
'
CMD$ = "SOURCE MEMORY3"
LENGTH% = LEN(CMD$)
CALL IOOUTPUTS(SCOPE&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'output the preamble
'
CMD$ = "PREAMBLE " + PRE$
LENGTH% = LEN(CMD$)
CALL IOOUTPUTS(SCOPE&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'output the data.  Note that if your device can use IEEE 488.2 arbitrary
'   block format, the commands from here down to right after we turn
'   EOL and EOI back on can be replaced with one IOOUTPUTAB command.
'
'turn off EOL and EOI when sending this next group of data as the next
' three output commands are really simulating one IEEE '#A' format
' transfer
'
CALL IOEOL(ISC&, CRLF$, DISABLE%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
CALL IOEOI(ISC&, DISABLE%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'first send the ascii #A
'
CMD$ = "DATA #A"
LENGTH% = LEN(CMD$)
CALL IOOUTPUTS(SCOPE&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'next, send the length (2 bytes)
'
LENGTH% = 2: FLAG% = 2
CALL IOOUTPUTB(SCOPE&, SEG BYTES%, LENGTH%, FLAG%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'now send the actual data
'
LENGTH% = BYTES%: FLAG% = 2
CALL IOOUTPUTB(SCOPE&, SEG READINGS%(1), LENGTH%, FLAG%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'turn EOL and EOI back on
'
CALL IOEOL(ISC&, CRLF$, ENABLE%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
CALL IOEOI(ISC&, ENABLE%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'display memory 3 on graph 2
'
CMD$ = "DISPLAY FORMAT 2;RECALL GRAPH 2, MEMORY 3;RESUME"
LENGTH% = LEN(CMD$)
CALL IOOUTPUTS(SCOPE&, CMD$, LENGTH%)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
'give local control back to the scope
'
CALL IOLOCAL(SCOPE&)
IF PCIB.ERR <> NOERR THEN ERROR PCIB.BASERR
'
SYSTEM
